#include <boost/lexical_cast.hpp>
#include <boost/assign/std/vector.hpp>
#include "genx.h"
#include <string>
#include <iostream>

/*
  g++ -I/afs/cern.ch/sw/lcg/external/Boost/1.32.0/slc3_ia32_gcc323/include/
  boost-1_32 -I../genx/ itAlignment.cpp ../genx/libgenx.a -o itAlignment
*/

using namespace boost::assign;
using namespace boost;

typedef const unsigned char* gTYPE;

genxWriter w = genxNew(NULL, NULL, NULL);
genxStatus status;

void oops(genxWriter w)
{
  fprintf(stderr, "oops ERROR %s\n", genxLastErrorMessage(w));
  exit(1);
}

extern void makeAlignmentCondition( const std::string& detectorName );

int main(){

  std::string detector = "IT";
  std::vector<std::string> station;
  station += "T1", "T2", "T3";
  std::vector<std::string> box;
  box += "Top", "Bottom", "Left", "Right";
  std::vector<std::string> layer;
  layer += "X1", "U", "V", "X2";
  std::vector<std::string> ladder;
  ladder += "Ladder1", "Ladder2", "Ladder3", "Ladder4",
            "Ladder5", "Ladder6", "Ladder7";

  genxElement dddb;
  genxElement catalog;
  genxElement conditionref;
  genxAttribute name;
  genxAttribute href;

  if (!(dddb =genxDeclareElement(w, NULL, (gTYPE)"DDDB", &status))) oops(w);
  if (!( conditionref = genxDeclareElement(w, NULL, (gTYPE)"conditionref", 
                                           &status))) oops(w);
  if (!( catalog = genxDeclareElement(w, NULL, (gTYPE)"catalog", &status)))
    oops(w);
  if (!(name = genxDeclareAttribute(w, NULL,(gTYPE) "name", &status))) oops(w);
  if (!(href = genxDeclareAttribute(w, NULL,(gTYPE) "href", &status))) oops(w);

  genxStartDocFile(w, stdout);
  genxStartElement(dddb);
    genxAddText(w, (gTYPE)"\n");

      // Condition for whole detector
      makeAlignmentCondition( detector + "System"); 

      // Loop over stations
      for ( int i = 0; i != 3; ++i ) {
        makeAlignmentCondition( detector + station[i] );
      }
      
      // Loop over stations
      for ( int i = 0; i != 3; ++i ) {
        // Loop over boxes
        for ( int j = 0; j != 4; ++j ) {
          makeAlignmentCondition( detector + station[i] + box[j] + "Box" );
        }
      }
      
      // Loop over stations
      for ( int i = 0; i != 3; ++i ) {
        // Loop over boxes
        for ( int j = 0; j != 4; ++j ) {
          // Loop over layers
          for ( int k = 0; k != 4; ++k ) {
                makeAlignmentCondition( detector + station[i] + box[j] + "Layer"
                                        +layer[k] );
          }
        }
      }

      // Loop over stations
      for ( int i = 0; i != 3; ++i ) {
        // Loop over boxes
        for ( int j = 0; j != 4; ++j ) {
          // Loop over layers
          for ( int k = 0; k != 4; ++k ) {
            // Loop over ladders
            for ( int l = 0; l != 7; ++l ) {
                makeAlignmentCondition( detector + station[i] + box[j] + "Layer"
                                        +layer[k] + ladder[l] );
            }
          }
        }
      }
  genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndDocument(w);

  std::cout << " " << std::endl;
  std::cout << std::endl;


  genxStartDocFile(w, stdout);
  genxStartElement(dddb);
    genxAddText(w, (gTYPE)"\n");
    genxStartElement(catalog);
      genxAddAttribute(name, (gTYPE)detector.c_str());
      genxAddText(w, (gTYPE)"\n  ");

      // Condition for whole detector
      genxStartElement(conditionref);
        std::string ref = "Alignment/Detectors.xml#"+ detector + "System";
        genxAddAttribute(href,(gTYPE) ref.c_str()); 
      genxEndElement(w);genxAddText(w, (gTYPE)"\n  ");

      // Loop over stations
      for ( int i = 0; i != 3; ++i ) {
        genxStartElement(conditionref);
          ref = "Alignment/Detectors.xml#"+ detector + station[i];
          genxAddAttribute(href,(gTYPE) ref.c_str()); 
        genxEndElement(w);genxAddText(w, (gTYPE)"\n  ");
      }
      
      // Loop over stations
      for ( int i = 0; i != 3; ++i ) {
        // Loop over boxes
        for ( int j = 0; j != 4; ++j ) {
          genxStartElement(conditionref);
            ref = "Alignment/Detectors.xml#"+ detector + station[i] + box[j] 
              + "Box";
            genxAddAttribute(href,(gTYPE) ref.c_str()); 
          genxEndElement(w);genxAddText(w, (gTYPE)"\n  ");
        }
      }
      
      // Loop over stations
      for ( int i = 0; i != 3; ++i ) {
        // Loop over boxes
        for ( int j = 0; j != 4; ++j ) {
          // Loop over layers
          for ( int k = 0; k != 4; ++k ) {
            genxStartElement(conditionref);
              ref = "Alignment/Detectors.xml#"+ detector + station[i] + box[j] 
                + "Layer" + layer[k];
              genxAddAttribute(href,(gTYPE) ref.c_str()); 
            genxEndElement(w);genxAddText(w, (gTYPE)"\n  ");
          }
        }
      }

      // Loop over stations
      for ( int i = 0; i != 3; ++i ) {
        // Loop over boxes
        for ( int j = 0; j != 4; ++j ) {
          // Loop over layers
          for ( int k = 0; k != 4; ++k ) {
            // Loop over ladders
            for ( int l = 0; l != 7; ++l ) {
              genxStartElement(conditionref);
                ref = "Alignment/Detectors.xml#"+ detector + station[i] 
                  + box[j] + "Layer" + layer[k] + ladder[l];
                genxAddAttribute(href,(gTYPE) ref.c_str()); 
              genxEndElement(w);genxAddText(w, (gTYPE)"\n  ");
            }
          }
        }
      }


    genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndElement(w); genxAddText(w, (gTYPE)"\n");
  genxEndDocument(w);

  std::cout << " " << std::endl;
  std::cout << std::endl;

  return 0;
}

extern void makeAlignmentCondition( const std::string& detectorName )
{
  genxElement paramVector;
  genxElement condition; 
  genxAttribute name;
  genxAttribute type; 
  genxAttribute classID;

  if (!(paramVector=genxDeclareElement(w, NULL, (gTYPE)"paramVector", &status)))
    oops(w);
  if (!( condition = genxDeclareElement(w, NULL, (gTYPE)"condition", 
                                           &status))) oops(w);
  if (!(name = genxDeclareAttribute(w, NULL,(gTYPE) "name", &status))) oops(w);
  if (!(classID = genxDeclareAttribute(w, NULL,(gTYPE) "classID", &status)))
    oops(w);
  if (!(type = genxDeclareAttribute(w, NULL,(gTYPE) "type", &status))) oops(w);

  genxStartElement(condition);
  genxAddAttribute(classID,(gTYPE) "6");
  genxAddAttribute(name, (gTYPE)detectorName.c_str());
  genxAddText(w, (gTYPE)"\n    ");
    genxStartElement(paramVector);
    genxAddAttribute(name,(gTYPE)"dPosXYZ");
    genxAddAttribute(type,(gTYPE)"double");
    genxAddText(w,(gTYPE)"0 0 0");
    genxEndElement(w);
    genxAddText(w, (gTYPE)"\n    ");
    genxStartElement(paramVector);
    genxAddAttribute(name,(gTYPE)"dRotXYZ");
    genxAddAttribute(type,(gTYPE)"double");
    genxAddText(w,(gTYPE)"0 0 0");
    genxEndElement(w);
    genxAddText(w, (gTYPE)"\n  ");
  genxEndElement(w);
  genxAddText(w, (gTYPE)"\n  ");
        
}
